
const REGION = 'us-east-1'
const CUMTD_API_KEY = 'insert_api_key_here'

var https = require('https')
const AWS = require('aws-sdk')
const docClient = new AWS.DynamoDB.DocumentClient({region: REGION})

exports.handler = (event, context, callback) => {

    // ----- DynamoDB functions ----- //

    // get the default/saved bus stop for this user
    var readDefaultBus = function(callback) {
        var userId = event.session.user.userId;
        var defaultStop = false;

        var params = {
            TableName: 'busStopDefaults',
            Key: {
                "userId": userId
            }
        };

        docClient.get(params, function(err, data) {
            if (!err && data && data.Item && data.Item.stopId) {
                defaultStop = data.Item.stopId;
            }

            return callback(defaultStop);
        })
    };

    // save the stop id in dynamodb to be saved as default
    var saveStopId = function(stopId, callback) {
        var params = {
            Item: {
                userId: event.session.user.userId,
                stopId: stopId
            },
            TableName: 'busStopDefaults'
        }

        docClient.put(params, function(err, data) {
            if (err) {
                console.log(err);
            }

            return callback();
        });
    };

    // ---- bus stop functions ----- //

    // get json of all stops - concise listing
    var getAllStops = function(callback) {
        return JSON.parse(`{"roland and comanche": "RLNDCMCHE", "broadmoor and crescent": "BDMRCRSNT", "university and fourth": "UNI4TH", "romine and tremont": "RMNTRMNT", "bluegrass and brookfield": "BLGRSBRKFLD", "goodwin at ceramics building": "GDWNCB", "state and maple": "STEMPL", "dodson dr. east and slayback": "DODSLYBK", "dart": "DART", "first and st. mary's rd.": "1STSMR", "main and lierman": "MNLRMN", "coventry and roxford": "CVTRYRXFRD", "cherry hills and coppertree": "CHHLCPRTRE", "springer and country fair dr.": "SGRCFD", "moraine and lake shore dr": "MRANLSD", "dodson and washington": "DDSNWASH", "vine and mckinley": "VINMCNLY", "broadway and crystal lake dr.": "BDWYCLD", "race and sherwin": "RACESWND", "randolph and springfield": "RNDLFSPFLD", "mattis and princeton": "MATSPRNCTN", "windsor and monterey": "WDSRMTRY", "eastern and kerr": "EASTKER", "mckinley and eureka": "MCNLYERKA", "brownfield and agnes": "BNFLDAGNS", "bardeen and toalson": "BRDNTLSN", "green and urbana": "GRNUBNA", "philo and fairlawn": "PHILOFRLN", "galen and devonshire": "GLNDVN", "williamsburg and mattis": "WMBGMATS", "cobblefield and englewood": "CBLFLDEGLWD", "fieldstone at horizon hobby": "FLDSTNHH", "pebblebrook and bluegrass": "PBLBRKBLGRS", "windsor and willowbrook ct.": "WDSRWWBK", "market and kenyon rd.": "MKTKNYN", "illinois and glenn dr.": "ILGLN", "mchenry and anderson": "MHRYADRSN", "susan stone and jane ann ct.": "SSTNJACT", "farley and west ridge": "FRLYWRDG", "state and charles": "STECHAS", "boardwalk and meijer dr.": "BRDWLKMJR", "wright and chalmers": "WRTCHAL", "market st. and garwood": "MKTGAR", "cunningham and park": "CNHMPK", "mullikin and doverbrook": "MLKNDVRBRK", "university and randolph": "UNIRNDLF", "university at mtd": "UNIMTD", "first and gregory": "1STGRG", "kirby and ridge rd.": "KBYRDGRD", "cobblefield and glenn brooke": "CBLFLDGLNBRK", "harris and alabama": "HRISAL", "anthony and dale": "ANTDALE", "pennsylvania and ag. engineering": "PAAG", "mchenry and pond": "MHRYPND", "main and urbana": "MNUBNA", "university and maple": "UNIMPL", "duncan and crossing ct.": "DNCNCRSNG", "lincoln and college ct.": "LNCLNCC", "harris and maple": "HRISMPL", "interstate dr. at walmart": "INTWALMART", "sherwin and yankee ridge": "SWNDYNKR", "kirby and alma": "KBYALMA", "balboa and prospect": "BLBOAPSPCT", "mckinley and maple": "MCNLYMPL", "springfield and highland": "SPFLDHILND", "mchenry and cottage grove": "MHRYCTGRV", "mattis and sumac": "MATSSMC", "windsor and myra ridge": "WDSRMYRA", "market place": "MKTPLC", "george huff and race": "GHFRACE", "harris and bradley": "HRISBRAD", "cunningham and oakland": "CNHMOKLN", "anderson and sunnycrest": "ADRSNSNYC", "curtis meadow and brighton": "CTSMDWBRTN", "mullikin and graves ct.": "MLKNGRVCT", "vine and george huff": "VINGHF", "florida and james cherry": "FLJASCHER", "park and romine": "PKRMN", "mattis and southwood": "MATSSTHWD", "atrium apts": "ATRIUM", "mchenry and lynn": "MHRYLYN", "brownfield and airport rd.": "BNFLDAPT", "market and logan": "MKTLGN", "winchester and blackthorn": "WNCHBKTRN", "staley and colleen": "STLYCLEEN", "university and ash": "UNIASH", "crescent and sangamon": "CRSNTSANG", "pennsylvania and broadway": "PABDWY", "fedex on killarney": "FEDEXA", "university and second": "UNI2ND", "elm and vine": "ELMVIN", "mullikin and crossgate": "MLKNCRSGT", "town center and moreland": "TCBMRLND", "fourth and kirby": "4THKBY", "florida and cottage grove": "FLCTGRV", "william and maplepark": "WLMMPLPK", "hancock and liberty": "HNKLBRTY", "trails and chatham": "TRLSCHAT", "vine and fairlawn": "VINFRLN", "cherry hills and curtis": "CHHLCTS", "maplepark and sangamon": "MPLPKSANG", "mattis and sangamon": "MATSSANG", "market and peach tree": "MKTPCH", "university and race": "UNIRACE", "windsor and crossing ct.": "WDSRCRSNG", "washington and macarthur": "WASHMCRTR", "bluegrass and summersage": "BLGRSSMRSG", "cottage grove and oregon": "CTGRVOR", "church and miller": "CHCHMLR", "bradley and carver": "BRADCRVR", "church and elm": "CHCHELM", "mattis and gentry square ln. south": "MATSGNSQ", "main and brady": "MNBRDY", "broadmoor and lakeside dr.": "BDMRLSDED", "neil and anthony dr.": "NEILANT", "lincolnshire and foothill": "LCNSRFTHL", "john and edwin": "JOHNEDWN", "michigan and green ridge": "MIGRNRDG", "neil and edgebrook": "NEILEGB", "galen and theodore": "GLNTDR", "lincoln and killarney": "LNCLNKLRNY", "st. mathews parking lot and lincolnshire": "LCNSR", "wright and springfield": "WRTSPFLD", "bentbrook and riverknoll": "BNTBRKRVRKNL", "myra ridge and silver hill": "MYRASLVHL", "neil and vine": "NEILVIN", "main and cottage grove": "MNCTGRV", "country club and willow": "CCRWLLW", "carrol fire department": "CROLFD", "gregory at mumford hall": "GRGMUM", "windsor and mullikin": "WNDSRMLKN", "timbergate and cherry hills": "TMGTCHHL", "randolph and tremont": "RNDLFTRMNT", "crestwood and bradley": "CRSTWDBRAD", "galen and wilshire": "GLNWLSH", "lyndhurst and burwash": "LNHSTBRWS", "vet-med": "VETMED", "clayton and duncan": "CLTNDNCN", "duncan and meadows west": "DNCNMWW", "race and washington #2": "RACEWASH2", "silver and fletcher": "SLVRFLCHR", "d.s.c. on clark": "DSCCLK", "mattis and cypress": "MATSCYPS", "windsor rd": "WDSR", "lincoln and florida": "LNCLNFL", "town center blvd and neil": "TCBNEIL", "main at ileas": "MNILEAS", "myra ridge and trails": "MYRATRLS", "main and chestnut": "MNCHSNT", "green and state": "GRNSTE", "cherry hills and lakeview": "CHHLLKVW", "bradley and neil": "BRADNEIL", "scottswood and nevada": "SCWDNV", "fifth and grove": "5THGRV", "main and webber": "MNWBR", "fairview and lincoln": "FRVWLNCLN", "randolph and columbia": "RNDLFCLMBA", "windsor and morrissey park dr.": "WDSRMYPK", "florida and orchard": "FLOD", "john and russell": "JOHNRSL", "staley and turnberry": "STLYTRNBRY", "perkins and cooks lane": "PKNSCOOK", "cherry hills and applewood": "CHHLAPLWD", "clayton and covington": "CLTNCVT", "harris and sherwood": "HRISSHWD", "neil and kenyon road": "NEILKNYN", "clayton and bellerieve": "CLTNBLRV", "green and orchard": "GRNORCH", "market place north entrance and market s": "MKTPLCNMKT", "fifth and chalmers": "5THCHAL", "green and second": "GRN2ND", "anderson and michigan": "ADRSNMI", "silver and vawter": "SLVRVWTR", "philo and colorado": "PHILOCO", "kirby and carriage way": "KBYCRGWY", "randolph and healey": "RNDLFHLY", "state and tremont": "STETRMNT", "bloomington at home depot": "BLOMHD", "green and cedar": "GRNCDR", "first and armory": "1STARY", "bradley and elm": "BRADELM", "market and roper": "MKTRPR", "kirby and carolyn": "KBYCAROL", "pennsylvania and maryland dr.": "PAMD", "john and draper": "JOHNDRPR", "jefferson and columbia": "JFSNCLMBA", "market place west": "MKTPLCW", "kirby and duncan": "KBYDNCN", "springfield and new": "SPFLDNEW", "main and lynn": "MNLYN", "crossgate and ironwood ln": "CRSGTIRNWD", "windsor and staley": "WNDSRSTLY", "kirby and stratford": "KBYSRTFRD", "university and elm": "UNIELM", "lyndhurst and arbours drive south": "LNHSTARBR1", "captiva and george": "CPTVAGRG", "market place at wilbur": "MKPLCWLB", "colorado at county market": "COCM", "st mary's road and neil": "SMRNEIL", "race and michigan": "RACEMI", "duncan and sierra": "DNCNSRA", "pennsylvania and goodwin": "PAGWN", "windsor and cherry hills": "WDSRCHHL", "windsor and fields south": "WNDSRFLDS", "randolph and charles": "RNDLFCHAS", "mattis at mattis north apartments": "MATSMNA", "wildflower and trailway": "WLDFLRTRLWY", "staley and inverness": "STLYINV", "sherwin and sherwin cir 2": "SWNDSWNC2", "race and nevada": "RACENV", "meijer drive and prospect": "MJRPSPCT", "illinois terminal": "IT", "cobblefield and bridge point dr": "CBLFLDBDGPT", "second and hill": "2NDHILL", "anthony and mattis": "ANTMATS", "williamsburg and summerlin": "WMBGSMLN", "country fair dr. at r.p. lumber": "CFDRP", "bradley and state": "BRADSTE", "university and willis": "UNIWLS", "wright and stoughton": "WRTSTOTN", "church and state": "CHCHSTE", "market st. and bradley": "MKTBRAD", "curtis road and south first st.": "CTS1ST", "university and fair": "UNIFAIR", "u of i credit union": "UICU", "mattis and william": "MATSWLM", "highcross and bruce acres": "HICRSBA", "galen and plymouth": "GLNPLMT", "trails and philo": "TRLSPHILO", "vine and oregon": "VINOR", "airport rd. and highcross": "APTHICRS", "university and mattis": "UNIMATS", "campbell and aztec": "CMBLAZTEC", "john and goldenview": "JOHNGLDVW", "michigan and smith": "MISMITH", "sixth and bradley": "6THBRAD", "main and glover": "MNGLVR", "duncan and southwood": "DNCNSWOOD", "anderson and burkwood": "ADRSNBKWD", "florida ave. residence hall": "FAR", "willow and beason": "WLLWBESN", "hunter and lanore": "HNTRLNR", "airport and donald": "ARPRTDNLD", "vine and water": "VINWTR", "lincoln and park": "LNCLNPK", "cunningham and crystal lake": "CNHMCLD", "willow and baker": "WILLOWBKR", "university and lynn": "UNILYN", "brittany trail and mullikin": "BRTTRLMLKN", "anthony and willow": "ANTHNYWLLW", "lincoln square": "LSE", "prospect and floral park": "PSPCTFLRPK", "neil and marketview": "NEILMKTV", "race and california": "RACECA", "state and avondale": "STEAVNDL", "nevada and gregory place": "NVGRGPL", "winchester and glenoak": "WNCHGLNK", "cottage grove and glenwood oaks": "CTGRVGNWD", "town and country apts.": "TCA", "goodwin and nevada": "GWNNV", "university and lake": "UNILAKE", "newport and milford": "NWPRTMLFRD", "lyndhurst and park lane": "LNHSTPKLN", "white and sixth": "WHT6TH", "country squire and kinch": "CSQRKNCH", "sixth and chalmers": "6THCHAL", "prospect and marketview": "PSPCTMKTV", "green and webber": "GRNWBR", "kirby and waverly": "KBYWAVE", "curtis meadow and foxborough": "CTSMDWFXBRGH", "paula and melanie": "PLAMELNE", "race and windsor": "RACEWDSR", "church and lynn": "CHCHLYN", "fifth and columbia": "5THCLMBA", "wildflower and chickory": "WLDFLRCKRY", "vine and pell cir.": "VINPELL", "u.s. 150 and dodson dr.": "150DOD", "florida at thomas paine": "FLTOMPAINE", "washington school": "WASHSCL", "scottswood and illinois": "SCWDIL", "duncan and clark": "DNCNCLKRD", "pomona and deske ct.": "PMNADSK", "pomona and bradley": "PMNABRAD", "savoy walmart": "SWALMART", "university and wright": "UNIWRT", "mckinley and harvard": "MCNLYHVAD", "kirby and valley rd.": "KBYVYRD", "randolph and maple": "RNDLFMPL", "kenyon and cracker barrel": "KNYNCRKRB", "perkins at ivanhoe": "PKNSIVAN", "galen and ross": "GLNROSS", "springfield and kenwood": "SPFLDKNWD", "hunter and austin dr.": "HNTRASTN", "neil and stadium": "NEILSTDM", "fox and knollwood": "FOXKLWD", "winchester and branch": "WNCHBRCH", "vine and illinois": "VINIL", "state and ells": "STEELS", "cyprus point and stillwater landing": "CPRSPTSTLWTR", "crescent and rodney": "CRSNTROD", "country fair dr. and university": "CFDUNI", "williamsburg between bradley & summerlin": "WMBG", "state and columbia": "STECLMBA", "carroll and kerr": "CROLKER", "bradley and hickory": "BRADHKRY", "green and grove": "GRNGRV", "orchard downs mid-block north": "ODMIDN", "anthony dr. at goodwill": "ANTGOODWILL", "windsor and prospect": "WDSRPSPCT", "lincoln and wascher": "LNCLNWSCR", "lakeshore dr. and maynard": "LSDMYNRD", "lyndhurst and old maple lane": "LNHSTOML", "state and beardsley": "STEBRD", "woodfield and curtis road": "WDFLDCTS", "pomona and heritage": "PMNAHRTG", "wedgewood and timbergate": "WDGWDTMGT", "lincoln and kettering park": "LNCLNKING", "main and art bartell rd.": "MNABD", "neil and bellefontaine": "NEILBELL", "duncan and daniel": "DNCNDNL", "john and prospect": "JOHNPSPCT", "brownfield and arlene": "BNFLDARL", "john and lynn": "JOHNLYN", "anderson and mumford": "ADRSNMFRD", "airport and landis farm": "ARPRTLNDSFRM", "wright and dublin": "WRTDBLN", "church and new": "CHCHNEW", "fairlawn and grant pl.": "FRLNGRNT", "mattis and devonshire": "MATSDVN", "susan stone and galena": "SSTNGLNA", "crestwood and ladue": "CRSTWDLDU", "green and fourth": "GRN4TH", "florida and philo": "FLPHILO", "crossgate and charwick": "CRSGTCHWK", "neil and marshall": "NEILMSHL", "duncan and clayton": "DNCNCLTN", "crescent and william": "CRSNTWLM", "main and vine": "MNVIN", "liberty and jefferson": "LBRTYJEF", "race and iowa": "RACEIA", "mattis and healey": "MTSHLY", "william and brentwood": "WLMBRNTWD", "vine and michigan": "VINEMI", "crestwood and clayton": "CRSTWDCLTN", "beech and romine": "BCHRMN", "vine and brookens": "VINBRKNS", "columbia and betsy ross": "CLMBABTSYRS", "florida and maple": "FLMPL", "florida and kinch": "FLKNCH", "susan stone and moss": "SSTNMOS", "susan stone and summit": "SSTNSMT", "randolph and hill": "RNDLFHIL", "main and ennis": "MNENIS", "highcross and apostalic life church": "HICRSALC", "windsor and village green": "WDSRVLGGRN", "green and sixth": "GRN6TH", "colbert park": "CBRTPK", "kirby and bellepark dr.": "KBYBLPK", "kirby and coronado": "KBYCRNDO", "butzow at guardian west": "GRD", "main and maple": "MNMPL", "kerr and berkley": "KERBRKLY", "paddock dr. east at winfield village lot": "PDKELOTB", "church and garfield": "CHCHGRFLD", "market and marketview": "MKTMKTV", "town center and menards": "TCBMNRDS", "fairview and goodwin": "FRVWGWN", "washington and lierman": "WASHLRMN", "springhill and glenhill": "SPGHLGLNHL", "green and anderson": "GRNADRSN", "fox at remax real estate": "FOXREMAX", "anderson and colorado": "ADRSNCO", "boardwalk and town center blvd": "BRDWLKTCB", "champaign house": "CMIHS", "john and mattis": "JOHNMATS", "vine and montclair": "VINMNTCLR", "anthony and dawn": "ANTHYDAWN", "vine and scovill": "VINSCVL", "captiva and northwood": "CPTVANRTHWD", "the place at 117": "INDIGO", "campbell and queensway": "CMBLQNWAY", "harding at steer place": "HRDGSTP", "john and ridgeway": "JOHNRGWAY", "randolph and vine": "RNDLFVIN", "green and johnson": "GRNJNSN", "wendover and country bend": "WNDVRCTRYBND", "john and dogwood": "JOHNDGWD", "kirby and arrow rd.": "KBYARW", "lyndhurst and bedford": "LNHSTBDFD", "sunnycrest shopping center": "SUNNYCREST", "mchenry and philo": "MHRYPHILO", "green and wright": "GRNWRT", "church and willis": "CHCHWLS", "green and prospect": "GRNPSPCT", "mattis and stonegate dr.": "MATSSTNGT", "church and james": "CHCHJAS", "fieldstone at metritech": "FLDSTNMETRI", "susan stone and myra ridge": "SSTNMYRA", "brownfield and prairieview": "BNFLDPRVW", "mattis and ivy ct. south": "MATSIVYS", "devonshire and sandpiper": "DVNSNPR", "fourth and bradley": "4THBRAD", "westfield and sangamon": "WSFLDSANG", "university and hickory": "UNIHKRY", "newton and nobel": "NWTNNBL", "market and bellefontaine": "MKTBELL", "university and neil": "UNINEIL", "randolph and beardsley": "RNDLFBRD", "kinch and easy": "KNCHEASY", "slayback and carrie": "SLYBKCRE", "mchenry and combes": "MHRYCMBS", "campbell and dobbins": "CMBLDBNS", "lawndale and bellepark": "LWNDLBLPK", "romine and bradley": "RMNBRAD", "university and goodwin": "UNIGWN", "park lane and bedford": "PKLNBDFD", "perkins and richard": "PKNSRICH", "balboa and southwest dr.": "BLBOASWD", "springfield and chicago": "SPFLDCHI", "church and russell": "CHCHRSL", "staley and clubhouse": "STLYCLBHS", "springfield and vandoren": "SPFLDVNDRN", "state and springfield": "STESPFLD", "broadway and franklin": "BDWYFRKLN", "second and church": "2NDCHCH", "winchester and cypress": "WNCHCYPS", "broadway and bailey": "BDWYBLY", "country club and division": "CCRDVSN", "jefferson middle school": "JMS", "macarthur and macarthur ct.": "MCRTRCT", "john and new": "JOHNNEW", "cottage grove and penn.": "CTGRVPA", "harris at franklin middle school": "HRISFMS", "round barn road and country fair dr.": "RBRCFD", "john and garfield": "JOHNGRFLD", "duncan and stoneybrook": "DNCNSTNBK", "green and mathews": "GRNMAT", "pennsylvania and dorner": "PADNR", "crescent and lawndale": "CRSNTLWNDL", "state and white": "STEWHT", "crescent and parkdale": "CRSNTPKDL", "main and smith rd.": "MNSMITH", "university and russell": "UNIRSL", "mchenry and lynn st. #2": "MHRYLYN2", "prospect and kent": "PSPCTKENT", "country fair dr. and church": "CFDCHCH", "anthony and north shore": "ANTNSHR", "john and highland": "JOHNHILND", "broadway and stebbins": "BDWYLST", "windsor at christie clinic": "WDSRCCS", "university and victor": "UNIVCTR", "kirby and kenwood": "KBYKNWD", "broadway and park": "BDWYPK", "anthony and walters": "ANTWTRS", "country brook": "CB", "beslin and romine": "BSLNRMN", "rolling acres and marjorie": "RLACRMRJ", "lyndhurst and arbours drive": "LNHSTARBR", "duncan and lawndale": "DNCNLWNDL", "prospect and ventura": "PSPCTVTRA", "bradley and redwood": "BRADRDWD", "springfield and ridgeway": "SPFLDRGWY", "urbana meijer": "URBMEIJ", "coventry and maynard": "CVTRYMYNRD", "bloomington and alabama": "BLOMAL", "cottage grove and washington": "CTGRVWASH", "church and neil": "CHCHNEIL", "john and prairie": "JOHNPRRE", "country fair dr. and camp": "CFDCAMP", "illinois street residence hall": "ISR", "market and paul": "MKTPAUL", "goodwin and clark": "GWNCLK", "country bend and summerhill": "CTRYBNDSUMHL", "central high east": "CE", "goodwin and church": "GWNCHCH", "first and hazelwood": "1STHZLWD", "mattis and glenn park dr.": "MATSGLNPK", "illini union": "IU", "switchgrass and trailway": "SWTCHGRSTRLWY", "fox drive at community elements": "FOXCE", "first and daniel": "1STDAN", "maplepark and william": "MPLPKWLM", "westfield and william": "WSFLDWLM", "second and park": "2NDPK", "fox and windsor": "FOXWDSR", "broadway and thompson": "BDWYTPSN", "paula and mattis": "PLAMATS", "mercury and gemini ct.": "MERCGEMCT", "randolph and avondale": "RNDLFAVNDL", "scottswood and oregon": "SCWDOR", "race and mumford": "RACEMUMFRD", "florida and linden": "FLLNDN", "winchester and southmoor": "WNCHSMR", "race at stone creek church": "RACEUA", "kirby and western": "KBYWSTRN", "roland and aztec": "RLNDAZTEC", "westfield and sheridan": "WSFLDSRDN", "market place and convenience drive": "MKTPLCCD", "broadmoor and david": "BDMRDVD", "country club and golfview": "CCRGFVW", "pebblebrook and rockdale": "PBLBRKRKDL", "neil and center": "NEILCNTR", "white and fourth": "WHT4TH", "lincoln and vermont": "LNCLNVT", "duncan and lager": "DNCNLGR", "anthony at rural king": "ANTHYRK", "st. mathews": "STMAT", "orchard downs north shelter": "ODNS", "mayfair and theodore": "MYFRTDR", "interstate and research": "INTRSRCH", "smith and carrie": "SMITHCRE", "anderson and harding": "ADRSNHRDG", "lyndhurst and evergreen circle": "LNHSTEGRN", "neil and north": "NEILN", "university and central": "UNICENT", "foothill and robert": "FTHLRBT", "market place at center drive": "NEILCD", "kirby and state": "KBYSTE", "roland and walters": "RLNDWTRS", "kirby and crescent": "KBYCRSNT", "lincoln and oregon": "LNCLNOR", "mayfair and weathervane": "MYFRWTRVN", "race and high st.": "RACEHIGH", "william and goldenview": "WLMGLDVW", "maynard and coventry": "MYNDCVTRY", "windsor west apts": "WDSRWEST", "restoration urban ministries": "RESTURBMIN", "willow and kenyon": "WILLOWKNYN", "gregory at library": "GRGLIB", "randolph and eureka": "RNDLFERKA", "anderson and silver": "ADRSNSLVR", "independent living": "ILAPTS", "park lane and stanford": "PKLNSTFD", "williamsburg and garden hills lane": "WMBGGH", "sherwin and sherwin cir.": "SWNDSWNC", "lincoln and church": "LNCLNCHCH", "paula and garden hills drive": "PLAGH", "john and fair": "JOHNFAIR", "washington and kinch": "WASHKNCH", "windsor and vine": "WDSRVIN", "prospect and eureka": "PSPCTERKA", "crescent and ola": "CRSNTOLA", "lot e-14": "E14", "one illinois north": "ONENORTH", "staley and kearns": "STLYKRNS", "independence and betsy ross": "INDPNCBTSYRS", "church and denton": "CHCHDNTN", "summersage and switchgrass": "SMRSGSWTCHGRS", "windsor and brookshire": "WDSRBKSR", "university and prairie": "UNIPRRE", "kirby and park haven": "KBYPRKHVN", "public health district on kenyon": "CUPHD", "crescent and greendale": "CRSNTGRNDL", "philo and scovill": "PHILOSCVL", "bonnie blair and gold medal": "BNBLRGDMD", "washington and timothy tr.": "WASHTIM", "mercury at fedex": "MERCFEDEX", "kirby and mattis": "KBYMATS", "broadway at save a lot": "BDWYCM", "brownfield and mary lou dr.": "BNFLDML", "cunningham and thompson": "CNHMTPSN", "bentbrook and pinecrest": "BNTBRKPNCRST", "university and prospect": "UNIPSPCT", "airport and willow": "ARPRTWLLW", "kirby and westfield": "KBYWSFLD", "florida and race": "FLRACE", "john and frank": "JOHNFRANK", "mtd garage": "DEPOT", "vine and lynn": "VINLYN", "springfield and country fair dr.": "SPFLDCFD", "philo and laurel": "PHILOLRL", "urbana walmart": "UWALMART", "broadway and university": "BDWYUNI", "washington and scottswood": "WASHSCWD", "town center at meijer": "TCBMJR", "bentbrook and cobblefield": "BNTBRKCBLFLD", "mckinley and beardsley": "MCNLYBRD", "john and pine": "JOHNPINE", "state and stanage": "STESTNAG", "walnut and logan": "WLNTLGN", "bradley and willis": "BRADWLS", "church and prairie": "CHCHPRRE", "church and busey": "CHCHBUSY", "neil and twins": "NEILTWN", "anthony and kankakee": "ANTKANK", "neil and hessel": "NEILHSL", "country fair dr. and cone": "CFDCONE", "john and duncan": "JOHNDNCN", "kirby and neil": "KBYNEIL", "springfield and garfield": "SPFLDGRFLD", "state and cedar": "STECDR", "anthony at napleton's": "ANTHNYNAP", "smith and rainbow view": "SMRNBVW", "william and oakcrest": "WLMOKCRST", "market st behind barnes and noble books": "MKTBNB", "race and main": "RACEMN", "prospect and columbia": "PSPCTCLMBA", "copper ridge and sand": "CPRRDGSND", "bloomington and prospect": "BLOMPSPCT", "fairlawn": "FRLN", "cherry hills and pine valley": "CHHLPNVY", "hamilton and jefferson": "HMTNJFSN", "main and grossbach": "MNGRSBCH", "florida and pleasant dr.": "FLPLSNT", "brittany trail and southford": "BRTTRLSTHFRD", "dodson and nevada": "DDSNNV", "vine and mchenry": "VINMHRY", "windsor and winchester": "WDSRWNCH", "coler and fairview": "CLRFRVW", "bloomington and sunset": "BLOMSNST", "country fair dr. at social security": "CFDSSI", "crescent at jefferson middle school": "CRSNTJMS", "university and harvey": "UNIHVY", "paula and jeanne": "PLAJEANNE", "kenyon at steak and shake": "KNYNSTKSHK", "galen and harrington": "GLNHRTN", "springfield and mckinley": "SPFLDMCNLY", "liberty and columbia": "LBRTYCLMBA", "duncan and ridgewood": "DNCNRDGWD", "perkins and highcross": "PKNSHICRS", "mattis and meadow dr.": "MATSMDWDR", "race and indiana": "RACEIN", "duncan and valleybrook": "DNCNVYBK", "cottage grove and mitchem": "CTGRVMCHM", "cottage grove and green": "CTGRVGRN", "vine and elm": "VINELM", "mullikin and copper ridge": "MLKNCPRRG", "bentbrook and bentbrook ct": "BNTBRKBBCT", "main and dewey": "MNDWY", "fifth and vine": "5THVIN", "farber and newton": "FRBRNWTN", "boulder ridge and farley": "BLDRRGFRLY", "windsor and carlisle": "WDSRCLSL", "one illinois south": "ONESOUTH", "cottage grove and colorado": "CTGRVCO", "harris and beardsley": "HRISBRD", "bradley and mattis": "BRADMATS", "copper ridge and mullikin": "CPRRDGMLKN", "gregory and dorner": "GRGDNR", "john and hollycrest": "JOHNHLYCST", "philo and harding": "PHILOHRDG", "main and broadway": "MNBRDWY", "john and westlawn": "JOHNWLWN", "white and second": "WHT2ND", "cottage grove and philo": "CTGRVPHILO", "newport and landis farm": "NWPRTLNDSFRM", "university and mccullough": "UNIMCLGH", "bradley and james": "BRADJAS", "pinecrest and porterfield": "PNCRSTPRTFLD", "hessel and randolph": "HSLRNDLF", "park and mathews": "PKMAT", "vine and mumford": "VINMFRD", "staley and kirby": "STLYKBY", "oak and gerty": "OAKGTY", "broadway and kerr": "BDWYKER", "state and clark": "STECLARK", "church and victor": "CHCHVCTR", "florida and anderson": "FLADRSN", "mayfair and georgetown": "MYFRGTOWN", "wendover and woodspring": "WNDVRWDSPG", "fields south and farhills": "FLDSFRHL", "pennsylvania and busey": "PABUSEY", "harris and harvard": "HRISHRVRD", "state and eureka": "STEERKA", "park and wright": "PKWRT", "queensway": "QNWAY1", "slayback and mcgee": "SLYBKMG", "campbell and roland": "CMBLRLND", "pennsylvania at par": "PAPAR", "skyline and newport": "SKYLNNWPRT", "woodfield and arbour town place": "WDFLDABT", "vine and sunnycrest": "VINSNYC", "guardian dr. and butzow": "GRDBTZ", "anthony at ameren": "ANTAMRN", "first and stadium": "1STSTDM", "lincoln and sunset": "LNCLNSNST", "meijer drive at meijer": "MEIJER", "mattis and alton": "MATSALTN", "anderson and brighton": "ADRSNBRTN", "bluegrass and boca vista": "BLGRSBCAVSTA", "hamilton and betsy ross": "HMLTNBTSYRS", "florida and hillcrest": "FLHLCRST", "wright and beslin": "WRTBSLN", "bradley and mckinley": "BRADMCNLY", "pomona and ed hoffman": "PMNAEH", "devonshire and devonshire ct.": "DVNDVNCT", "prospect at meijer's ent.": "PSPCTMJR", "campbell and curt": "CMBLCRT", "springfield and gregory st.": "SPFLDGRGST", "springfield and russell": "SPFLDRSL", "farhills and staley": "FRHLSTLY", "st. mary's road and oak": "SMROAK", "state and hessel blvd.": "STEHSL", "farber at cerl": "FRBRCERL", "viking south": "VKINGS", "oak and anthony": "OAKANT", "sixth and gregory": "6THGRG", "pennsylvania at law school": "PALAW", "william and kenwood": "WLMKNWD", "springfield and draper": "SPFLDDRPR", "illinois terminal at market st.": "ITMKT", "lincoln and bradley": "LNCLNBRAD", "napleton and anthony": "NPLTNANTHNY", "kirby and rose dr.": "KBYROSE", "sixth and armory": "6THARY", "stone creek and stillwater landing": "STNCRKSTLWTR", "kerr and cunningham": "KERCNHM", "bradley and clayton": "BRADCLTN", "kenyon at eastland suites": "KNYNESTLND", "church and prospect": "CHCHPSPCT", "fourth and st. mary's": "4THSMR", "kirby and lincoln rd.": "KBYLCLNRD", "springfield and duncan": "SPFLDDNCN", "green and neil": "GRNNEIL", "carroll at carroll addition community center": "CACC", "prospect and sherwood terrace": "PSPCTSHWD", "burwash and woodfield": "BRWSWDFLD", "coronado and dover": "CRNDODVR", "wright and columbia": "WRTCLMBA", "sunnycrest mall": "FLSUN", "columbia and hickory": "CLMBAHKRY", "washington and broadway": "WASHBDWY", "hunter and country squire": "HNTRCSQR", "bonnie blair and nancy beth": "BNBLRNNBTH", "mullikin and villa": "MLKNVLA", "cottage grove and california": "CTGRVCA", "wright and hill": "WRTHIL", "race and montclair": "RACEMTCLR", "maplepark and maplewood": "MPLPKMPLWD", "mchenry and vawter": "MHRYVWTR", "clayton and higgins": "CLTNHGN", "clayton and high view": "CLTNHV", "springfield and prairie": "SPFLDPRRE", "brittany trail and staley": "BRTTRLSTLY", "university and mckinley": "UNIMCNLY", "gregory at ikenberry commons": "GRGIKE", "prospect and dennison": "PSPCTDNSN", "florida and smith": "FLSMITH", "colorado at steer place": "STP", "mckinley and tremont": "MCNLYTRMNT", "winchester and barberry": "WNCHBRBY", "anthony at wood spring suites": "ANTHYWSS", "prospect and ayrshire cir.": "PSPCTASRC", "country squire ct.": "CSQRCT", "windward and autumnwood": "WINDATMWD", "green and goodwin": "GRNGWN", "state and hill": "STEHIL", "crescent and round barn manor": "CRSNTRBM", "marketview and anthony": "MKTVANT", "randolph and washington": "RNDLFWASH", "vine and green": "VINGRN", "airport and fieldcrest": "ARPRTFDCRST", "broadway and chapin": "BDWYCHPN", "springfield and fieldstone": "SPFLDFLDSTN", "copper ridge and blackhawk": "CPRRDGBLKHK", "duncan and valerie": "DNCNVLR", "windsor and donnybrook": "WDSRDNBK", "randolph and clark": "RNDLFCLARK", "goodwin and gregory": "GWNGRG", "columbia and hancock": "CLMBAHNK", "green and busey": "GRNBUSEY", "kirby and glenshire": "KRBYGLNSHR", "cottage grove and harding": "CTGRVHRDG", "peabody at krannert art museum": "PDYKAM", "state and trade center dr.": "STETC", "krannert center": "KRANNERT", "race and whitehall ct.": "RACEWTHL", "mullikin and trostshire": "MLKNTRSTSHR", "romine and eads": "RMNEADS", "pennsylvania ave. residence hall": "PAR", "ward and kerr": "WRDKER", "john and kenwood": "JOHNKNWD", "neil and town center apt.": "NEILTCA", "church and draper": "CHCHDRPR", "state and john": "STEJOHN", "mattis at post office": "MATSUSPS", "philo and briarcliff": "PHILOBCLF", "brownfield and johnson": "BNFLDJHN", "mattis and branch rd.": "MATSBRCH", "prospect and francis": "PSPCTFRNS", "kinch and rainbow view": "KNCHRAIN", "cottage grove and eastern": "CTGRVERN", "maplepark and lawndale": "MPLPKLWNDL", "kirby and grandview dr. w.": "KBYGDVWW", "windsor and duncan": "WDSRDNCN", "hazelwood and oak": "HZLWDOAK", "plastipak": "PLASTIPAK", "florida and lincoln": "FLLNCLN", "anderson and fairlawn": "ADRSNFRLN", "aldi at rt. 150": "ALDI150", "airport and skyline": "ARPRTSKYLN", "cobblefield and inverness": "CBLFLDINV", "wright and healey": "WRTHLY", "paula and joanne lane": "PLAJOANE", "neil and arcadia": "NEILARC", "main and rt. 150": "MN150", "williamsburg and winston": "WMBGWNTN", "philo and silver": "PHILOSLVR", "bloomington and louisiana": "BLOMLA", "windsor and lynn": "WDSRLYN", "curtis meadow and staley": "CTSMDWSTLY", "springfield at phillips rec. ctr.": "SPFLDPRC", "copper ridge and ironwood": "CPRRDGIRNWD", "broadmoor and galen": "BDMRGLN", "silver and cottage grove": "SLVRCTGRV", "windsor and stillwater": "WDSRSTLW", "race and meadow": "RACEMDW", "staley and copper ridge": "STLYCPRRDG", "convenience dr. and market": "CNVMKT", "elm and park": "ELMPK", "prospect and galen": "PSPCTGALN", "pomona and santa ana": "PMNASTANA", "bradley at heritage park": "HRTGPK", "mullikin and stonebridge": "MLKNSTNBDG", "randolph and stanage": "RNDLFSTNAG", "washington and second": "WASH2ND", "lincoln and delaware": "LNCLNDE", "anderson and evergreen circle": "ADRSNEVRGN", "brownfield and columbia": "BNFLDCLMBA", "fairview and busey": "FRVWBUSEY", "nevada and lincoln": "NVLNCLN", "main and scottswood": "MNSCWD", "prospect and broadmoor": "PSPCTBDMR", "pennsylvania and anderson": "PAADRSN", "market st. and mercury dr.": "MKTMERC", "boardwalk and interstate": "BRDWLKINT", "race and washington #1": "RACEWASH1", "winchester and broadmoor": "WNCHBDMR", "florida and adams": "FLADAMS", "ironwood and staley": "IRNWDSTLY", "fourth and pennsylvania": "4THPA", "prospect and healey": "PSPCTHEALY", "coronado and ventura": "CRNDOVTRA", "harris and bloomington": "HRISBLOM", "mattis and interstate": "MATSINT", "race and delaware": "RACEDE", "airport and somerset": "ARPRTSMRST", "university and sabin": "UNISBN", "church between new and prospect": "CHCH", "market place south": "MKTPLCS", "fairlawn and eastern": "FRLNERN", "prospect and tremont": "PSPCTTRMNT", "windward and countryside": "WINDCNTYSD", "mercury and apollo dr.": "MERCAPLO", "washington and hickory": "WASHHKRY", "country bend and springhill": "CTRYBNDSPGHL", "third and daniel": "3RDDAN", "windsor and lakewood": "WDSRLKWD", "amber and philo": "AMBPHILO", "windsor and susan stone": "WDSRSSTN", "sussex and oakmont": "SSXOKMNT", "crestwood and florence": "CRSTWDFLNC", "duncan and rolling acres": "DNCNRLACR", "big lots": "BL", "park and central": "PKCNTRL", "sherwin and pond ridge": "SWNDPNDR", "prospect and washington": "PSPCTWASH", "bradley and garden hills": "BRADGH", "race at meadowbrook park": "RACEMBRK", "perkins and cherokee": "PKNSCHKE", "kirby and mullikin": "KRBYMLKN", "crescent and john": "CRSNTJOHN", "mattis and anita": "MATSANTA", "captiva and sanibel": "CPTVASNBL", "fairview and harvey": "FRVWHRVY", "neil and columbia": "NEILCLMBA", "pennsylvania and wiley dr.": "PAWILEY", "florida and lincolnwood": "FLLCLNWD", "vine and burkwood ct. w.": "VINBKWD", "airport and arlene": "ARPRTARLNE", "bradley and willliamsburg": "BRADWMBG", "roland and dobbins": "RLNDDBNS", "alpine and timberline": "ALPTMBLN", "fourth between kirby & st marys": "4THBIELFELDT", "neil and beardsley": "NEILBRD", "race and holmes": "RACEHLMS", "devonshire center": "DCC", "paula and larkspur": "PLALRKSPR", "bloomington and willis": "BLOMWLS", "church and carson": "CHCHCRSN", "first at ashton woods": "ASHTON", "race and mchenry": "RACEMHRY", "lakeview and wedgewood": "LKVWWGWD", "randolph and university": "RNDLFUNI", "campus circle": "CAMPUSCIR", "duncan and prairie meadow": "DNCNPMDW", "plant sciences lab": "PSL", "cobblefield and crail": "CBLFLDCRL", "springfield and elm": "SPFLDELM", "county satellite jail": "CCSJ", "macarthur and high": "MCRTRHI", "church and edwin": "CHCHEDWN", "anthony and campbell": "ANTCMBL", "clayton and pomona": "CLTNPMNA", "race and oregon": "RACEOR", "kerr and ward": "KERWRD", "florida and vine": "FLVIN", "u.s. 150 and university": "150UNI", "fairview and linview": "FRVWLNVW", "crestwood and edward hoffman": "CRSTWDEDHFMN", "country club rd. at apts.": "CCRAPTS", "kirby and dodds": "KBYDODD", "prospect and devonshire": "PSPCTDVN", "town center and prospect": "TCBPSPCT", "holiday inn": "HDYINN", "prospect and john": "PSPCTJOHN", "pennsylvania ct.": "PACT", "green and prairie": "GRNPRRE", "lincoln and pennsylvania": "LNCLNPA", "parkland college": "PKLN", "peppermill and pebblebrook": "PPRMLPBLBRK", "vine and harris": "VINHRIS", "kirby and scottsdale": "KBYSCTDL", "halifax and maynard": "HLFXMYNRD", "green and maple": "GRNMPL", "fifth and tremont": "5THTRMNT", "pennsylvania and vine": "PAVIN", "winchester and aspen": "WNCHASP", "pinecrest and staley": "PNCRSTSTLY", "bluegrass and sandstone": "BLGRSSNDSTN", "florida and florida ct.": "FLFLCT", "market and wallace": "MKTWLACE", "farley and pebblebrook": "FRLYPBLBRK", "interstate and farber": "INTFRBR", "mchenry and fletcher": "MHRYFLCHR", "ginger creek apts.": "GC", "country fair west entrance": "CFW", "john and western": "JOHNWSTRN", "urbana high school": "UHS", "cottage grove and brighton": "CTGRVBRITN", "sussex and southfield": "SSXSFLD", "wendover and summerview": "WNDVRSMRVW", "crossgate and crossgate ct.": "CRSGTCRSGTCT", "stephens family ymca": "YMCA", "campbell and comanche": "CMBLCMCHE", "neil and washington": "NEILWASH", "paula and honeysuckle": "PLAHNYSKL", "prospect and hill": "PSPCTHILL", "john and willis": "JOHNWILS", "activities and recreation": "ARC", "pennsylvania and orchard": "PAORCH", "lierman and hunter": "LRMNHNTR", "springfield and country fair": "SPFLD", "anthony and dusk": "ANTHYDUSK", "kerr and geraldine": "KERGRLDNE", "lyndhurst and lyndhurst village court": "LNHSTLNVCT", "windsor and jordan": "WDSRJDN", "country fair dr. and park": "CFDPK", "washington and phillips": "WASHPHLPS", "fifth and beardsley": "5THBRD", "fairview and gregory": "FRVWGRG", "mchenry and lynn st. #1": "MHRYLYN1", "bloomington and mittendorf": "BLOMMTDF", "vine and prairie": "VINPRRE", "galen and yorkshire": "GLNYKSH", "first and southeast lot e-14": "1STE14", "clayton and creve coeur": "CLTNCRVCR", "vermont and kinch": "VTKNCH", "philo and mumford": "PHILOMFRD", "liberty and betsy ross": "LBRTYBTSYRS", "prospect and harvard": "PSPCTHVD", "neil and interstate": "NEILINTR", "amber and myra ridge": "AMBMYRA", "clayton and hilltop": "CLTNHLT", "switchgrass and wildflower": "SWTCHGRSWLDFLWR", "vine and washington": "VINWASH", "university and lierman": "UNILRMN", "main and fremont": "MNFREMNT", "williamsburg and thornton": "WMBGTRNTN", "urbana middle school": "UMS", "cunningham and perkins": "CNHMPKNS", "kettering and linview": "KPDLNVW", "main and walnut": "MNWLNT", "prospect and park lane": "PSPCTPKLN", "washington and rainbow view": "WASHRAIN", "bloomington and hagan": "BLOMHGN", "baytowne and boardwalk": "BAYBRDWLK", "chemical and life sciences": "CHEMLS", "race and vermont": "RACEVT", "mayfair and broadmoor": "MYFRBDMR", "country fair dr. at fluid events": "CFDFLDEV", "prospect and kirby": "PSPCTKBY", "neil and garwood": "NEILGAR", "galen and newbury": "GLNNWBY", "neil and springfield": "NEILSPFLD", "florida and rutledge": "FLRTLG", "florida and wiley": "FLWLY", "mayfair and old farm": "MYFROF", "bradley and hagan": "BRADHGN", "springfield and pine": "SPFLDPINE", "florida and florida dr.": "FLFLDR", "goodwin and park": "GWNPK", "winchester and tamarack": "WNCHTMRK", "viking north": "VKINGN", "washington and third": "WASH3RD", "highcross and kyle": "RE", "springfield and lynn": "SPFLDLYN", "windsor and melrose": "WDSRMLRS", "mullikin and remington way": "MLKNRMGNTN", "washington and ash": "WASHASH", "country fair": "CF", "dodson and illinois": "DDSNIL", "windsor and bel air": "WDSRBA", "washington and smith": "WASHSMITH", "springfield and edwin": "SPFLDEDWN", "brownfield and sharlyn": "BNFLDSLYN", "vine and colorado": "VINCO", "market and tulip tree": "MKTTLP", "green and lynn": "GRNLYN", "green and birch": "GRNBRCH", "slayback and ira": "SLYBKIRA", "state and washington": "STEWASH", "bonnie blair and amy": "BNBLRAMY", "lincoln and hill": "LNCLNHIL", "philo and michigan": "PHILOMI", "prospect and green": "PSPCTGRN", "fifth and eureka": "5THERKA", "bradley at wlrw": "WLRW", "mullikin and ironwood": "MLKNIRNWD", "trails and fawn hill": "TRLSFWN", "st. mary's road and griffith": "SMRGRIF", "walnut and columbia": "WLNTCLMBA", "windsor and valleybrook": "WDSRVYBK", "kerr and highlands": "KERHGHLNDS", "lincoln and illinois": "LNCLNIL", "mchenry & lynn - east": "MHRYLYNE", "duncan and blair": "DNCNBLR", "kinch and lantern hill": "KNCHLH", "market and wilbur": "MKTWLBR", "hagan and dennison": "HGNDNSN", "paula and powers": "PLAPWRS", "bloomington rd. at pioneer plaza": "BLOMPNR", "winchester and sumac": "WNCHSMC", "mayfair and lincolnshire": "MYFRLCLNSH", "john and foley": "JOHNFOLY", "airport and clifton": "ARPRTCLFTN", "kirby and rosewood": "KBYRSWD", "romine and eureka": "RMNERKA", "slayback and smith rd.": "SLYBKSMITH", "lincoln and st. marys rd": "LNCLNSMR", "park and mccullough": "PKMCCLGH", "pennsylvania and sixth": "PA6TH", "illinois employment training center": "IETC", "westfield and carrelton": "WSFLDCRLTN", "fox and devonshire": "FOXDVN", "central high": "CENTRAL", "fourth and chalmers": "4THCHAL", "marketview at sams": "MKTVSAMS", "lawndale and centennial": "LWNDLCENT", "state and birch": "STEBRCH", "mayfair and sandpiper": "MYFRSNFR", "broadway and country club": "BDWYCCR", "curtis and meadow": "CTSMDW", "prospect and lincolnshire": "PSPCTLNSH", "springfield and royal ct.": "SPFLDRYLCT", "broadway and oakland": "BDWYOKLN", "interstate and newmark": "INTNWMK", "northwood and george": "NRTHWDGRG", "galen and salisbury": "GLNSLBY", "kirby and cambridge": "KBYCMBG", "bonnie blair and sharon": "BNBLRSHRN", "mattis and round barn rd.": "MATSRBR", "university and draper": "UNIDRPR", "brownfield and perkins": "BNFLDPKNS", "romine and ellis": "RMNELLIS", "orchard downs mid-block south": "ODMIDS", "alpine and bradley": "ALPBRAD", "smith rd. and u.s. 150": "SMITH150", "springfield and birch": "SPFLDBRCH", "neil and hill st.": "NEILHIL", "bentbrook and staley": "BNTBRKSTLY", "fox drive and state": "FOXSTE", "harris and louisiana": "HRISLA", "lincoln at japan house": "JPN", "winchester and belmore": "WNCHBLMR", "market place southeast": "MKTPLCSE", "lynn and park": "LYNPRK", "mattis and harbor point": "MATSHRBR", "devonshire and noel": "DVNNL", "trails and baronry": "TRLSBRN", "paula and mckinley": "PLAMCNLY", "state and buena vista": "STEBV", "edison middle school": "EMS", "university and new": "UNINEW", "windsor and boulder": "WDSRBLDR", "westgate apts.": "WESTGATE", "devonshire and peach": "DVNPCH", "randolph and ells": "RNDLFELS", "kirby and dawson": "KBYDWSN", "linview and killarney": "LNVWKLRNY", "state and vine": "STEVIN", "anderson and eliot": "ADRSNELIOT", "anthony and dobbins": "ANTDOBB", "randolph and white": "RNDLFWHT", "windsor and imperial": "WDSRIMP", "fourth and armory": "4THARY", "cottage grove and illinois": "CTGRVIL", "coler and church": "CLRCHCH", "vine and holmes": "VINHLMS", "valerie and meaghan": "VLRMGHN", "mattis and henry st.": "MATSHNRY", "state and healey": "STEHLY", "fox at christie clinic": "FOXCHRSTIE", "crescent and southwood": "CRSNTSWD", "highcross and nordland": "HICRSNDLD", "neil and white": "NEILWHT", "brittany trail and southford trace west": "BRTTRLSTHFRDTR", "newton and newmark": "NWTNNWMK", "broadmoor and robert": "BDMRRBT", "broadmoor and mcdonald": "BDMRMCD", "mattis and broadmoor": "MATSBDMR", "winchester and brookhaven": "WNCHBKHVN", "walnut and washington": "WLNTWASH", "myra ridge and autumn ridge": "MYRAAUTM", "first at ihotel": "1STIHOTEL", "devonshire and lakeside dr.": "DVNLSDED", "wright and church": "WRTCHCH", "staley and amherst": "STLYAMHT", "randolph and john": "RNDLFJOHN", "tatman and rt. 130": "TAT130", "airport and cunningham": "ARPRTCNGHM", "state and university": "STEUNI", "crescent and round barn road": "CRSNTRBR", "mattis and bloomington": "MATSBLOM", "windsor at clark lindsey village": "WDSRCLV", "duncan and crestridge": "DNCNCSTRG", "windsor and scottsdale": "WDSRSCTDL", "sussex and waterford": "SSXWTRFD", "green and randolph": "GRNRNDLF", "perkins and carroll": "PKNSCROL", "kirby and greencroft": "KBYGRNCFT", "slayback and dodson west": "SLYBKDOD", "john and mitchell ct.": "JOHNMCHLCT", "bradley and crispus": "BRADCRPS", "cottage grove and crestwood": "CTGRVCRWD", "pennsylvania and carle": "PACARL", "parkland ct. and bradley": "PKLNCTBRAD", "broadmoor and georgetown": "BDMRGTWN", "dorner and peabody": "DNRPDY", "philo and windsor": "PHILOWDSR", "florida and busey": "FLBUSEY", "bradley at dsc": "DSC", "westfield and lawndale": "WSFLDLWNDL", "cobblefield and balmoral": "CBLFLDBLML", "lar": "LAR", "florida and curtiss": "FLCRTS", "prospect and interstate": "PSPCTINTER", "neil and healey": "NEILHLY", "prospect and pioneer": "PSPCTPNR", "randolph and william": "RNDLFWLM", "broadmoor and lincoln pl.": "BDMRLCNPL", "curtis and wynstone": "CRTSWNSTN", "springfield and staley": "SPFLDSTLY", "town center blvd at target": "TCBTGT", "leggett and bardeen": "LGGTBRDN", "mattis and ivy ct. north": "MATSIVYN", "goodwin at materials research": "GDWNMRL", "cobblefield and aberdeen": "CBLFDABDN", "bradley and thompson park": "TMPSNPK", "windward and waterford": "WINDWTRFD", "springfield and mattis": "SPFLDMATS", "pennsylvania and kinch": "PAKNCH", "springfield and busey": "SPFLDBSY", "williamsburg and summit ridge": "WMBGSTRG", "florida and grove": "FLGRV", "galen and sussex": "GLNSSX", "clayton and heritage": "CLTNHTG", "lyndhurst and magnolia ct.": "LNHSTMAGN", "clayton and irvine": "CLTNIRVN", "bloomington and highland": "BLOMHILND", "illinois and broadway": "ILBDWY", "white and first": "WHT1ST", "bradley and champaign st.": "BRADCMI", "bradley between pomona and wlrw": "BRAD", "church and mckinley": "CHCHMCNLY", "bradley and prospect": "BRADPSPCT", "wright and beech": "WRTBCH", "dover and balboa": "DVRBLBOA", "springfield and westlawn": "SPFLDWLWN", "bloomington and mckinley": "BLOMMCNLY", "mattis and sheridan": "MATSSHRDN", "springfield and james": "SPFLDJAS", "bradley and duncan": "BRADDNCN", "stone creek and venturi": "STNCRKVNTRI", "kerr and division": "KERDVSION", "perkins and raintree woods": "PKNSRNTRW", "philo and pennsylvania": "PHILOPA", "logan and first": "LGN1ST", "crestridge and lakewood": "CRSTRGLKWD", "vine and shurts": "VINSRTS", "prospect and vine": "PSPCTVIN", "white and wright": "WHTWRT", "neil and walingford": "NEILWLGFRD", "kirby and mayfair": "KBYMYFR", "neil and eureka": "NEILERKA", "fox drive at surs": "FOXSURS", "pennsylvania and race": "PARACE", "scottswood and calif. ct.": "SCWDCACT", "fifth and washington": "5THWASH", "fifth and bradley": "5THBRAD", "champaign walmart": "WALMART", "stadium and oak": "STDMOAK", "butzow and lierman": "BTSLRMN", "oak between hazelwood and st. mary's": "OAK", "main and poplar": "MNPLR", "william and duncan": "WLMDNCN", "hagen and pioneer": "HGNPNR", "devonshire and mayfair": "DVNMYFR", "devonshire and lynnwood dr.": "DVNLNWD", "brownfield and clifton": "BNFLDCFTN", "john and victor": "JOHNVCTR", "prospect and maple": "PSPCTMPL", "sixth and peabody": "6THPDY", "newton and research": "NWTNRSRCH", "romine and beardsley": "RMNBRD", "kirby and oak": "KBYOAK", "stone creek and bohn": "STNCRKBN", "interstate and waterview way": "INTERSTWW", "kerr and carroll": "KERCROL", "colorado and larch": "COLRCH", "william and stratford": "WLMSTFRD", "transit plaza": "PLAZA", "boulder ridge and peppermill": "BLDRRDGPPRML", "market place north entrance": "MKTPLCN", "oak and kenyon": "OAKKNYN", "sherwin and persimmon": "SWNDPSMN", "state and william": "STEWLM", "vance and napleton": "VNCENPLTN", "windward point": "WINDPT", "main and dodson": "MNDDSN", "inverness and prestwick pt": "INVPSTWK", "university and vine": "UNIVIN", "bluegrass and boulder ridge": "BLGRSBLDRRDG", "hancock and hamilton": "HNKHMTN", "brady and high": "BRDYHI", "fourth and peabody": "4THPDY", "mullikin and westborough": "MLKNWSTBRGH", "green and gregory st.": "GRNGRGST", "wright and washington": "WRTWASH", "university and sixth": "UNI6TH", "lincoln and anthony": "LNCLNANT", "church and fair": "CHCHFAIR", "washington and fourth": "WASH4TH", "john and james": "JOHNJAS", "clayton and crestview": "CLTNCSV", "prospect and ayrshire dr.": "PSPCTASRD", "valerie and alpine": "VLRALPN", "brittany trail and kensington knoll": "BRTTRLKNTNKNL", "clayton and compton": "CLTNCPT", "fourth and gregory": "4THGRG", "armory and wright": "ARYWRT", "airport and captiva": "ARPRTCPTVA", "prospect and beardsley": "PSPCTBRD", "paula and kirk lane": "PLAKIRK", "springfield and harvey": "SPFLDHVY", "university and ambucs park entrance": "AMBUC", "first and gerty": "1STGTY", "anthony and columbia": "ANTHYCLMBA", "fairlawn and cottage grove": "FRLNCTGRV", "pennsylvania and fourth": "PA4TH", "copper ridge and copper": "CPRRDGCPR", "marketview dr. at marketview center": "MKTV", "washington and country squire": "WASHCSQR", "vine and evergreen ct. w": "VINEGRN", "goodwin and main": "GWNMN", "washington and lanore": "WASHLNR", "main and grove": "MNGRV", "john and chicago": "JOHNCHI", "paula and cynthia": "PLACYNTHA", "bradley and walnut": "BRADWLNT", "queensway and roland": "QNWAYRLND", "water survey": "ISWS", "green and locust": "GRNLCST", "sussex and windward": "SSXWIND", "macarthur and california": "MCRTRCA", "mattis and windsor": "MATSWDSR", "university and james": "UNIJAS", "first and lake park south": "1STLAKEPKS", "john and elm": "JOHNELM", "main and hartle": "MNHTL", "u.s. 150 and dale": "150DALE", "research at hobbico": "RSRCHHBCO", "newton and leggett": "NWTNLGGT", "clayton and delmar": "CLTNDLMR", "state and park": "STEPK", "florida and bermuda": "FLBRMD", "florida and carle": "FLCARLE", "william and bellepark": "WLMBLPRK", "paddock dr. west at winfield village lot": "PDKWLOTM", "church and mattis": "CHCHMATS", "curtis road at winfield village": "CTSWIN", "crestwood and katie lynn": "CRSTWDKTLYN", "lincoln and eads": "LNCLNEADS", "broadmoor and lynwood": "BDMRLNWD", "race and illinois": "RACEIL", "william and centennial": "WLMCENT", "macarthur and illinois": "MCRTRIL", "university and sycamore": "UNISCMR", "bentbrook and golf creek": "BNTBRKGLFCRK", "perkins and eastern": "PKNSEAST", "toalson and interstate": "TLSNINTER", "lakeview and worthington": "LKVWWTGTN", "first and lake park north": "1STLAKEPKN", "bradley and country fair drive": "BRADCFD", "clayton and rossmoor": "CLTNRSM", "green and race": "GRNRACE", "randolph and bradley": "RNDLFBRAD", "susan stone and plains ct.": "SSTNPLN", "florida and broadway": "FLBRDWY", "vine and willis": "VINWLS", "i.e.t.c. and mattis": "IETCMATS", "westfield and alton": "WSFLDALTN", "mattis and green": "MTSGRN", "cherry hills and crestridge": "CHHLSCSTRG", "university and cottage grove": "UNICTGRV", "michigan and kinch": "MIKNCH", "mattis and chevy chase": "MATSCHEVY", "lierman and bartell": "LRMNBRTL", "orchard downs south shelter": "ODSS", "wendover and pebble creek": "WNDVRPBLCK", "clayton and clearwater": "CLTNCLW", "eastern": "EAST", "windsor and windward blvd.": "WDSRWIND", "goodwin and hill": "GWNHIL", "first and windsor": "1STWDSR", "fairlawn and patton": "FRLNPTN", "broadmoor and bellamy": "BDMRBLMY", "parkview": "PARKVIEW", "mullikin and curtis meadow": "MLKNCTSMDW", "neil and burwash": "NEILBRWS", "springfield and prospect": "SPFLDPSPCT", "carroll between community center and ker": "CROL", "windsor and phinney": "WDSRFNY", "galen and windsor": "GLNWDSR", "john and mckinley": "JOHNMCNLY", "bonnie blair and valerie": "BNBLRVLR", "church and orchard": "CHCHORCD", "walnut and university": "WLNTUNI", "springfield and willis": "SPFLDWLS", "kirby and cobblefield": "KBYCBLFLD", "myra ridge and ridge park": "MYRARGPK"}`);
    };

    var getDeparturesForStop = function(stopName) {

        // first get all of the stops to search for the right one
        stops = getAllStops();

        // go through each possible stop name and see if there is a match
        var possibleStopNames = getStopPermuations(stopName);
        var stopId = false;
        for (stopName of possibleStopNames) {
            if (!stops[stopName]) {
                continue;
            }

            // found match, get ID and make API call, then build string
            stopId = stops[stopName];
            break;
        }

        if (!stopId) {
            return sendTextResponse(context, `I'm sorry, I could not find a stop called ${stopName}`);
        }

        saveStopId(stopId, () => {
            getDepartures(stopId);
        });
    };

    // call the endpoint for departures given stopId, then write response
    var getDepartures = function(stopId) {
        if (!stopId) {
            return sendTextResponse(context, "No stop ID passed in");
        }

        var endpoint = `https://developer.cumtd.com/api/v2.2/JSON/GetDeparturesByStop?key=${CUMTD_API_KEY}&stop_id=${stopId}&pt=20`
        var body = ""

        https.get(endpoint, (response) => {
            response.on('data', (chunk) => {body += chunk})
            response.on('end', () => {
                var data = JSON.parse(body);
                var departures = data.departures;
                writeResponse(departures);
            })
        })
    };

    // will return an array of possible names for this input stop
    var getStopPermuations = function(stopName) {

        var possibleNamesForStop = [stopName];

        // if the key is found in the stop name, add the converted value to posssible names
        var conversions = {
            'road': 'rd.', 'road': 'rd',
            'street': 'st.', 'street': 'st',
            'first': '1st', '1st': 'first',
            'second': '2nd', '2nd': 'second',
            'third': '3rd', '3rd': 'third',
            'fourth': '4th', '4th': 'fourth',
            'fifth': '5th', '5th': 'fifth',
            'sixth': '6th', '6th': 'sixth'
        };

        // if the stopname is called '<x> and <y>', create a permutation called '<y> and <x>'
        if (stopName.split(' ').length === 3 && stopName.split(' ')[1] === 'and') {
            possibleNamesForStop.push(stopName.split(' ')[2] + ' and ' + stopName.split(' ')[0])
        }

        // apply any conversion
        for (var conversion in conversions) {
            var startingLength = possibleNamesForStop.length;
            for (var i = 0; i < startingLength; i++) {
                if (possibleNamesForStop[i].indexOf(conversion) > -1) {
                    var permutation = possibleNamesForStop[i].replace(conversion, conversions[conversion])
                    possibleNamesForStop.push(permutation)
                }
            }
        }

        // convert all to lower case
        possibleNamesForStop = possibleNamesForStop.map( (x) => x.toLowerCase());

        return possibleNamesForStop;
    };

    // write the response to Alexa
    var writeResponse = function(departures) {
        if (!departures || departures.length === 0) {
            return sendTextResponse(context, "There are no buses arriving at this time")
        } else {

            var responseText = "In the next 20 minutes, there is";

            for (var i = 0; i < departures.length; i++) {
                var departure = departures[i];
                var route_number = departure['route']['route_short_name']
                var direction = departure['trip']['direction']
                var expected_mins = departure['expected_mins']

                if (i !== 0 && i === (departures.length - 1)) {
                    responseText += ' and'
                }

                if (Number(expected_mins) > 0) {
                    responseText += ` a ${route_number} ${direction} arriving in ${expected_mins} minute`
                    if (Number(expected_mins) !== 1) responseText += 's'
                } else {
                    responseText += ` a ${route_number} ${direction} arriving now`
                }

                if (i !== (departures.length - 1)) {
                    responseText += ','
                }
            }

            return sendTextResponse(context, responseText)
        }
    };

    // cancel the session
    var endSession = function() {
        sendTextResponse(context, "Goodbye");
    }

    // provide help to the user
    var helpSession = function() {
        sendTextResponse(context, "Get the next scheduled buses for a stop by asking Alexa. An example is, 'Alexa, get bus times for first and daniel'. What bus stop would you like?", 'continue');
    }

    // switch and case on the request type, take appropriate action
    switch(event.request.type) {

        case "LaunchRequest":
            readDefaultBus((stopId) => {
                if (!stopId) {
                    return sendTextResponse(context, "You have no default stop, what bus stop would you like?", 'continue')
                }
                getDepartures(stopId)
            });
            break;

        case "IntentRequest":

            if (event.request.intent.name === "AMAZON.HelpIntent") {
                return helpSession();
            }

            // get the stop requested, if it exists, false otherwise
            if (event.request.intent.slots && event.request.intent.slots['Stop']) {
                stopRequested = event.request.intent.slots['Stop'].value

                if (stopRequested.toLowerCase() == "cancel" || stopRequested.toLowerCase() == "stop") {
                    return endSession();
                }

                if (stopRequested.toLowerCase() == "help") {
                    return helpSession();
                }

                getDeparturesForStop(stopRequested)
            } else {
                sendTextResponse(context, "Sorry, there was no stop given")
            }
            break;

        case "SessionEndedRequest":
            break;

        default:
            context.fail(`Invalid request type: ${event.request.type}`)
    }

};

// Helpers

buildSpeechletResponse = (outputText, shouldEndSession) => {
    return {
        outputSpeech: {
            type: "PlainText",
            text: outputText
        },
        shouldEndSession: shouldEndSession
    }
}

generateResponse = (speechletResponse, sessionAttributes) => {
    return {
        version: "1.0",
        sessionAttributes: sessionAttributes,
        response: speechletResponse
    }
}

sendTextResponse = (context, responseText, shouldEndSession) => {

    if (shouldEndSession === 'continue') {
        shouldEndSession = false;
    } else {
        shouldEndSession = true;
    }

    context.succeed(
        generateResponse(
            buildSpeechletResponse(responseText, shouldEndSession),
            {}
        )
    )
}
